/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite s_HBOffs_HBOff0,
 * Boston, MA 02111-1s_HBOff07, USA.
 */


#include "TFPreviewDialog.h"
#include "GuiColormap.h"
#include "GuiColormapBands.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFPreviewDialog::TFPreviewDialog (HeightField *HF, HeightFieldDraw *HFD, 
				  char *winTitle, char *paramTitle,
				  int tableX, int tableY) 
	        : GuiDialogPreview (winTitle, paramTitle, tableX, tableY),
		  d_vBoxOptions (TRUE, 5)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFPreviewDialog::TFPreviewDialog");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFPreviewDialog::TFPreviewDialog");

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFPreviewDialog\n");

	d_cx = d_cy = -1;
	d_hfTimestamp = 0;
	d_HF = NULL;
	d_HFD = NULL;
	d_HFO = NULL;
	d_HFPreview = NULL;
	d_HFDPreview = NULL;
	d_HFOPreview = NULL;
}


/*
 *  destructor: clean up 
 */
TFPreviewDialog::~TFPreviewDialog ()
{
	if (d_HFO)
		delete d_HFO;
	if (d_HFPreview)
		delete d_HFPreview;
	if (d_HFDPreview)
		delete d_HFDPreview;
	if (d_HFOPreview)
		delete d_HFOPreview;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFPreviewDialog\n");
}


/*
 *  setHBobjs: set the HF objects we operate on and generate a scaled 
 * 	down version for the preview 
 */
void TFPreviewDialog::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFPreviewDialog::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFPreviewDialog::setHFobjs");

	GuiColormap 	*cMap = HFD->getColormap();

	if (HF!=d_HF || HFD!=d_HFD)
		{
		if (d_HFO)
			delete d_HFO;
		d_HF = HF;
		d_HFD = HFD;
		d_HFO = new HeightFieldOps (d_HF);
		}

	if (d_HFPreview)
		delete d_HFPreview;
	if (d_HFDPreview)
		delete d_HFDPreview;
	if (d_HFOPreview)
		delete d_HFOPreview;
	d_HFPreview = d_HFO->hfResize (100, 100);
	d_HFDPreview = new HeightFieldDraw (d_HFPreview, this->d_drawArea);
	d_HFDPreview->setMode ("0");
	d_HFDPreview->setColormap (cMap);
	d_HFOPreview = new HeightFieldOps (d_HFPreview);
	d_HFPreview->backup ();
	d_hfTimestamp = d_HF->getUpdateTime();
}


/* 
 * drawCrosshairs: draw horizonal/vertical crosshairs
 */
void TFPreviewDialog::drawCrosshairs (int cx, int cy)
{
	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFPreviewDialog::drawCrosshairs");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFPreviewDialog::drawCrosshairs");

	int		w = d_drawArea->width(),
			h = d_drawArea->height();

	this->d_drawArea->setColors (&(this->d_HFDPreview->getColormap()->d_black));
	this->d_drawArea->drawLine (cx, 0, cx, h);
	this->d_drawArea->drawLine (0, cy, w, cy);
	d_drawArea->sync ();
}


/* 
 * previewUpToDate(): check if the preview HF is current (ie: if the master 
 *	HF has not been updated)
 */
void TFPreviewDialog::previewUpToDate ()
{
	SanityCheck::bailout ((d_HF->getUpdateTime()<d_hfTimestamp), "d_HF->getUpdateTime()<d_hfTimestamp", "TFPreviewDialog::previewUpToDate");

	if (d_HF->getUpdateTime() == d_hfTimestamp)
		return;

	setHFobjs (d_HF, d_HFD);
}


/*
 *  (virtual) checkboxCallbackPreview: process the 'use preview' checkbox 
 */
void TFPreviewDialog::checkboxCallbackPreview ()
{
	if (!d_cbUsePreview.get_active())
		{
		d_HFPreview->restore ();
		d_HFDPreview->draw ();
		}
	else
		{
		SanityCheck::bailout ((!d_HFDPreview), "HFDPreview==NULL", 
				"GuiDialogPreview::checkboxCallbackPreview");
		d_drawArea->set_sensitive (TRUE);	
		updatePreviewCallback ();
		d_HFDPreview->draw ();
		if ( (d_cx != -1) && (d_cy != -1) )
			drawCrosshairs (d_cx, d_cy);
		}
}


